using System;
using System.Collections.Generic;
using System.Text;
using AoLib.Utils;

namespace VhaBot.Plugins
{
    public class OnlineBase : PluginBase
    {
        private bool IncludeNotifyList = true;
        private bool IncludePrivateChannel = true;
        private bool SeperateSections = false;
        private string DisplayOrganization = "Never";
        private bool DisplayRanks = true;
        private string DisplayMode = "Profession";
        private bool DisplayHeaders = true;

        public OnlineBase()
        {
            this.Name = "Online Tracker";
            this.InternalName = "vhOnline";
            this.Author = "Vhab";
            this.Version = 100;
            this.Description = "Provides an interface to the user to view who is online and/or on the private channel.";
            this.DefaultState = PluginState.Installed;
            this.Commands = new Command[] {
                new Command("online", true, UserLevel.Guest, UserLevel.Member, UserLevel.Guest)
            };
        }

        public override void OnLoad(BotShell bot)
        {
            bot.Configuration.Register(ConfigType.Boolean, this.InternalName, "include_notify", "Include Notify List", this.IncludeNotifyList);
            bot.Configuration.Register(ConfigType.Boolean, this.InternalName, "include_pg", "Include Private Channel", this.IncludePrivateChannel);
            bot.Configuration.Register(ConfigType.Boolean, this.InternalName, "seperate_sections", "Seperate Sections", this.SeperateSections);
            bot.Configuration.Register(ConfigType.String, this.InternalName, "display_org", "Display Organization", this.DisplayOrganization, "Never", "Foreign", "Always");
            bot.Configuration.Register(ConfigType.Boolean, this.InternalName, "display_ranks", "Display Ranks", this.DisplayRanks);
            bot.Configuration.Register(ConfigType.String, this.InternalName, "display_mode", "Display Mode", this.DisplayMode, "Profession", "Alphabetical", "Level");
            bot.Configuration.Register(ConfigType.Boolean, this.InternalName, "display_headers", "Display Sub-Headers", this.DisplayHeaders);

            this.IncludeNotifyList = bot.Configuration.GetBoolean(this.InternalName, "include_notify", this.IncludeNotifyList);
            this.IncludePrivateChannel = bot.Configuration.GetBoolean(this.InternalName, "include_pg", this.IncludePrivateChannel);
            this.SeperateSections = bot.Configuration.GetBoolean(this.InternalName, "seperate_sections", this.SeperateSections);
            this.DisplayOrganization = bot.Configuration.GetString(this.InternalName, "display_org", this.DisplayOrganization);
            this.DisplayRanks = bot.Configuration.GetBoolean(this.InternalName, "display_ranks", this.DisplayRanks);
            this.DisplayMode = bot.Configuration.GetString(this.InternalName, "display_mode", this.DisplayMode);
            this.DisplayHeaders = bot.Configuration.GetBoolean(this.InternalName, "display_headers", this.DisplayHeaders);

            bot.Events.ConfigurationChangedEvent += new ConfigurationChangedHandler(Events_ConfigurationChangedEvent);
        }

        public override void OnUnload(BotShell bot)
        {
            bot.Events.ConfigurationChangedEvent -= new ConfigurationChangedHandler(Events_ConfigurationChangedEvent);
        }

        private void Events_ConfigurationChangedEvent(BotShell bot, ConfigurationChangedArgs e)
        {
            if (e.Section.Equals(this.InternalName, StringComparison.CurrentCultureIgnoreCase))
                switch (e.Key.ToLower())
                {
                    case "include_notify":
                        this.IncludeNotifyList = (bool)e.Value;
                        break;
                    case "include_pg":
                        this.IncludePrivateChannel = (bool)e.Value;
                        break;
                    case "seperate_sections":
                        this.SeperateSections = (bool)e.Value;
                        break;
                    case "display_org":
                        this.DisplayOrganization = (string)e.Value;
                        break;
                    case "display_ranks":
                        this.DisplayRanks = (bool)e.Value;
                        break;
                    case "display_mode":
                        this.DisplayMode = (string)e.Value;
                        break;
                    case "display_headers":
                        this.DisplayHeaders = (bool)e.Value;
                        break;
                }
        }

        public override void OnCommand(BotShell bot, CommandArgs e)
        {
            switch (e.Command)
            {
                case "online":
                    List<string> titles = new List<string>();
                    List<RichTextWindow> windows = new List<RichTextWindow>();
                    string args = string.Empty;
                    if (e.Args.Length > 0)
                        args = e.Args[0];

                    // Notify List
                    if (this.IncludeNotifyList)
                    {
                        string[] online = bot.FriendList.Online("notify");
                        if (online.Length > 0)
                        {
                            RichTextWindow notifyWindow = new RichTextWindow(bot);
                            notifyWindow.AppendHeader(online.Length + " Users Online");
                            int results = 0;
                            this.BuildList(bot, online, ref notifyWindow, ref results, args);
                            if (results > 0)
                            {
                                titles.Add(HTML.CreateColorString(bot.ColorHeaderHex, results.ToString()) + " Users Online");
                                windows.Add(notifyWindow);
                            }
                        }
                    }

                    // Private Channel
                    if (this.IncludePrivateChannel)
                    {
                        Dictionary<UInt32, Friend> list = bot.PrivateChannel.List();
                        List<string> guests = new List<string>();
                        foreach (KeyValuePair<UInt32, Friend> user in list)
                            guests.Add(user.Value.User);
                        if (guests.Count > 0)
                        {
                            RichTextWindow guestsWindow = new RichTextWindow(bot);
                            guestsWindow.AppendHeader(guests.Count + " Users on the Private Channel");
                            int results = 0;
                            this.BuildList(bot, guests.ToArray(), ref guestsWindow, ref results, args);
                            if (results > 0)
                            {
                                titles.Add(HTML.CreateColorString(bot.ColorHeaderHex, results.ToString()) + " Users on the Private Channel");
                                windows.Add(guestsWindow);
                            }
                        }
                    }

                    // Output
                    if (titles.Count < 1)
                    {
                        bot.SendReply(e, "The are currently no users online");
                        break;
                    }
                    if (SeperateSections)
                    {
                        for (int i = 0; i < windows.Count; i++)
                        {
                            RichTextWindow window = new RichTextWindow(bot);
                            window.AppendTitle();
                            window.AppendRawString(windows[i].Text);
                            bot.SendReply(e, titles[i] + "  ", window);
                        }
                    }
                    else
                    {
                        RichTextWindow window = new RichTextWindow(bot);
                        window.AppendTitle();
                        foreach (RichTextWindow subWindow in windows)
                            window.AppendRawString(subWindow.Text);
                        bot.SendReply(e, string.Join(", ", titles.ToArray()) + "  ", window);
                    }
                    break;
                default:
                    break;
            }
        }

        public void BuildList(BotShell bot, string[] users, ref RichTextWindow window) { int results = 0; this.BuildList(bot, users, ref window, ref results, string.Empty); }
        public void BuildList(BotShell bot, string[] users, ref RichTextWindow window, ref int results, string profs)
        {
            if (window == null)
                return;
            if (users.Length == 0)
                return;

            SortedDictionary<string, SortedDictionary<string, WhoisResult>> list = new SortedDictionary<string, SortedDictionary<string, WhoisResult>>();
            Dictionary<string, WhoisResult> whoisResults = new Dictionary<string, WhoisResult>();
            foreach (string user in users)
            {
                if (user == null || user == string.Empty)
                    continue;

                string header;
                WhoisResult whois = XML.GetWhois(user, bot.Dimension);
                if (profs != null && profs != string.Empty)
                {
                    if (whois == null || whois.Stats == null)
                        continue;
                    if (!whois.Stats.Profession.StartsWith(profs, StringComparison.CurrentCultureIgnoreCase))
                        continue;
                }
                if (whois != null && whois.Stats != null)
                {
                    if (this.DisplayMode == "Level")
                        header = whois.Stats.Level.ToString();
                    else
                        header = whois.Stats.Profession;

                    if (!whoisResults.ContainsKey(user.ToLower()))
                        whoisResults.Add(user.ToLower(), whois);
                }
                else
                    if (this.DisplayMode == "Level")
                        header = "0";
                    else
                        header = "Unknown";

                if (this.DisplayMode == "Alphabetical")
                    header = user.ToUpper().ToCharArray()[0].ToString();

                if (!list.ContainsKey(header))
                    list.Add(header, new SortedDictionary<string, WhoisResult>());
                list[header].Add(user, whois);
            }
            results = 0;
            foreach (KeyValuePair<string, SortedDictionary<string, WhoisResult>> prof in list)
            {
                if (this.DisplayHeaders)
                {
                    window.AppendHighlight(prof.Key);
                    window.AppendLineBreak();
                }
                foreach (KeyValuePair<string, WhoisResult> user in prof.Value)
                {
                    // Name
                    window.AppendNormalStart();
                    window.AppendString("   ");
                    int level = 0;
                    int ailevel = 0;
                    string name = Format.UppercaseFirst(user.Key);

                    // Level
                    if (user.Value != null && user.Value.Stats != null)
                    {
                        level = user.Value.Stats.Level;
                        ailevel = user.Value.Stats.DefenderLevel;
                    }
                    if (level < 10)
                        window.AppendColorString("000000", "00");
                    else if (level < 100)
                        window.AppendColorString("000000", "0");
                    window.AppendString(level + " ");

                    if (ailevel < 10)
                        window.AppendColorString("000000", "0");
                    window.AppendColorString(RichTextWindow.ColorGreen, ailevel + " ");
                    window.AppendString(name);

                    // Organization
                    bool displayOrganization = false;
                    if (this.DisplayOrganization == "Always")
                        displayOrganization = true;

                    if (this.DisplayOrganization == "Foreign")
                    {
                        displayOrganization = true;
                        if (bot.InOrganization && whoisResults.ContainsKey(name.ToLower()) && whoisResults[name.ToLower()].InOrganization && bot.Organization == whoisResults[name.ToLower()].Organization.Name)
                            displayOrganization = false;
                    }

                    if (displayOrganization)
                    {
                        if (!whoisResults.ContainsKey(name.ToLower()))
                        {
                            window.AppendString(" (Unknown)");
                        }
                        else if (whoisResults[name.ToLower()].InOrganization)
                        {
                            window.AppendString(" (" + whoisResults[name.ToLower()].Organization.ToString() + ")");
                        }
                        else
                        {
                            window.AppendString(" (Not Guilded)");
                        }
                    }

                    // Rank
                    if (this.DisplayRanks)
                    {
                        UserLevel userLevel = bot.Users.GetUser(name);
                        if (userLevel > UserLevel.Member)
                        {
                            window.AppendString(" (");
                            window.AppendColorString(RichTextWindow.ColorRed, bot.Users.GetUser(name).ToString());
                            window.AppendString(")");
                        }
                    }
                    window.AppendColorEnd();
                    window.AppendLineBreak();
                    results++;
                }
                if (this.DisplayHeaders)
                    window.AppendLineBreak();
            }
            if (!this.DisplayHeaders)
                window.AppendLineBreak();
        }
        public override string OnHelp(BotShell bot, string command)
        {
            switch (command)
            {
                case "online":
                    return "Displays all users online and/or on the private channel.\nIf [profession] is specified, it will display only people online and/or on the private channel that are the specified profession.\n" +
                        "Usage: /tell " + bot.Character + " online [[profession]]";
            }
            return null;
        }
    }
}
